module TypeScale exposing
    ( FontSizeLevel(..)
    , TypeScale
    , augmentedFourth
    , base
    , fontSizeMixin
    , fontSizePx
    , goldenRatio
    , h1
    , h2
    , h3
    , h4
    , majorSecond
    , majorThird
    , milli
    , minorSecond
    , minorThird
    , perfectFifth
    , perfectFourth
    , typeScale
    )

import Css
import Css.Global
import Responsive
    exposing
        ( CommonStyle
        , DeviceStyle
        , Mixin
        , ResponsiveStyle
        , lineHeight
        , mapMixins
        , mediaMixins
        , rhythm
        , styleAsMixin
        )



-- Type Scales.


type alias TypeScale =
    Int -> Float


typeScale : Float -> TypeScale
typeScale ratio n =
    ratio ^ toFloat (n - 1)


minorSecond : TypeScale
minorSecond =
    typeScale 1.067


majorSecond : TypeScale
majorSecond =
    typeScale 1.125


minorThird : TypeScale
minorThird =
    typeScale 1.2


majorThird : TypeScale
majorThird =
    typeScale 1.25


perfectFourth : TypeScale
perfectFourth =
    typeScale 1.333


augmentedFourth : TypeScale
augmentedFourth =
    typeScale 1.414


perfectFifth : TypeScale
perfectFifth =
    typeScale 1.5


goldenRatio : TypeScale
goldenRatio =
    typeScale 1.618


type FontSizeLevel
    = FontSizeLevel
        { level : Int
        , minLines : Int
        }


milli =
    FontSizeLevel
        { level = 0
        , minLines = 1
        }


base =
    FontSizeLevel
        { level = 1
        , minLines = 1
        }


h1 =
    FontSizeLevel
        { level = 5
        , minLines = 3
        }


h2 =
    FontSizeLevel
        { level = 4
        , minLines = 2
        }


h3 =
    FontSizeLevel
        { level = 3
        , minLines = 2
        }


h4 =
    FontSizeLevel
        { level = 2
        , minLines = 2
        }


fontSizePx : TypeScale -> DeviceStyle -> FontSizeLevel -> Float
fontSizePx scale { baseFontSize } (FontSizeLevel sizeLevel) =
    (scale sizeLevel.level * baseFontSize)
        |> floor
        |> toFloat


fontSizeMixin : TypeScale -> FontSizeLevel -> CommonStyle -> DeviceStyle -> Mixin
fontSizeMixin scale (FontSizeLevel sizeLevel) common device =
    let
        pxVal =
            fontSizePx scale device (FontSizeLevel sizeLevel)

        numLines =
            max sizeLevel.minLines
                (ceiling (pxVal / lineHeight common.lineHeightRatio device))
    in
    Css.batch
        [ Css.fontSize (Css.px pxVal)
        , Css.lineHeight (rhythm common device (toFloat numLines))
        ]
        |> styleAsMixin
